// $Id: gc_exportable.h,v 1.7 2011/06/22 18:22:22 jmcgill Exp $

/*
  Copyright 2002  Mary Kuhner, Jon Yamato, and Joseph Felsenstein

  This software is distributed free of charge for non-commercial use
  and is copyrighted.  Of course, we do not guarantee that the software
  works, and are not responsible for any damage you may cause or have.
*/

#ifndef GC_EXPORTABLE_H
#define GC_EXPORTABLE_H

#include <map>
#include <vector>

class GCIndividual;
class gcPopulation;
class gcRegion;

class gcPopRegionPair : public std::pair<const gcPopulation *, const gcRegion *>
{
  public:
    gcPopRegionPair(const gcPopulation *, const gcRegion *);
    virtual ~gcPopRegionPair();
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcNameResolvedInfo
{
  private:
    const gcPopulation &        m_populationRef;
    const gcRegion &            m_regionRef;
    std::vector<GCIndividual*>  m_individuals;

  public:
    gcNameResolvedInfo(const gcPopulation &, const gcRegion &);
    virtual ~gcNameResolvedInfo();

    void                                AddIndividual(GCIndividual*);
    const gcPopulation &                GetPopRef() const;
    const gcRegion &                    GetRegionRef() const;
    std::vector<const GCIndividual*>    GetIndividuals() const;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcExportable : public std::map<gcPopRegionPair,gcNameResolvedInfo *>
{
  public:
    gcExportable();
    ~gcExportable();
    const gcNameResolvedInfo &  GetInfo(const gcPopulation &, const gcRegion &) const;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

#endif  // GC_EXPORTABLE_H

//____________________________________________________________________________________
