// $Id: gc_migration.h,v 1.3 2012/05/01 21:37:47 ewalkup Exp $

/*
  Copyright 2002  Mary Kuhner, Jon Yamato, and Joseph Felsenstein

  This software is distributed free of charge for non-commercial use
  and is copyrighted.  Of course, we do not guarantee that the software
  works, and are not responsible for any damage you may cause or have.
*/

#ifndef GC_MIGRATION_H
#define GC_MIGRATION_H

#include "wx/string.h"
#include "gc_creation_info.h"
#include "gc_phase.h"
#include "gc_quantum.h"
#include "gc_types.h"

class GCStructures;

class gcMigration : public GCQuantum
{
    friend class GCStructures;

  private:
    bool                        m_blessed;  // true if user has edited m_nPanels

    bool                        m_hasFrom;
    size_t                      m_fromId;

    bool                        m_hasTo;
    size_t                      m_toId;
    
    double                      m_startValue;
    migration_method            m_method;
    migration_profile           m_profile;
    migration_constraint        m_constraint;
    
    gcCreationInfo              m_creationInfo;  //JRM this doing anything?
    
    void   SetCreationInfo(const gcCreationInfo &);
    
  public:

    gcMigration();
    ~gcMigration();

    void        SetBlessed(bool blessed);
    bool        GetBlessed()            const;

    bool        HasFrom()             const;
    void        SetFromId(size_t id);
    size_t      GetFromId()           const;
    wxString    GetFromIdString()     const;

    bool        HasTo()                const;
    void        SetToId(size_t id);
    size_t      GetToId()              const;
    wxString    GetToIdString()        const;
    
    void        SetStartValue(double val);
    double      GetStartValue()          const;
    wxString    GetStartValueString()    const;
    
    void                 SetMethod(migration_method val);
    migration_method     GetMethod()          const;
    wxString             GetMethodString()    const;
    
    void                 SetProfile(migration_profile val);
    migration_profile    GetProfile()          const;
    wxString             GetProfileAsString()    const;
    
    void                 SetConstraint(migration_constraint val);
    migration_constraint GetConstraint()          const;
    wxString             GetConstraintString()    const;


    void        DebugDump(wxString prefix=wxEmptyString) const;

};

#endif  // GC_MIGRATION_H

//____________________________________________________________________________________
