// $Id: gc_structure_maps.h,v 1.15 2011/12/01 22:32:42 jmcgill Exp $

/*
  Copyright 2002  Mary Kuhner, Jon Yamato, and Joseph Felsenstein

  This software is distributed free of charge for non-commercial use
  and is copyrighted.  Of course, we do not guarantee that the software
  works, and are not responsible for any damage you may cause or have.
*/

#ifndef GC_STRUCTURE_MAPS_H
#define GC_STRUCTURE_MAPS_H

#include <list>
#include <map>
#include <set>

#include "gc_locus.h"
#include "gc_migration.h"
#include "gc_panel.h"
#include "gc_parent.h"
#include "gc_population.h"
#include "gc_quantum.h"
#include "gc_region.h"
#include "gc_parent.h"
#include "gc_phenotype.h"
#include "gc_set_util.h"
#include "gc_trait.h"
#include "gc_trait_allele.h"

#include "gc_loci_match.h"
#include "gc_pop_match.h"

#include "wx/string.h"

class GCParseBlock;

class gcDisplayOrder :  public std::list<size_t>
{
  public:
    gcDisplayOrder() ;
    ~gcDisplayOrder() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
    wxString AsString() const;
};

class gcPopLocusIdPair : public std::pair<size_t, size_t>
{
  public:
    gcPopLocusIdPair() ;
    gcPopLocusIdPair(size_t i,size_t j);
    ~gcPopLocusIdPair() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcBlockSetMap : public std::map<gcPopLocusIdPair,gcIdSet>
{
  public:
    gcBlockSetMap() ;
    ~gcBlockSetMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcRegionMap : public std::map<size_t,gcRegion>
{
  public:
    gcRegionMap() ;
    virtual ~gcRegionMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcPopMap : public std::map<size_t,gcPopulation>
{
  public:
    gcPopMap() ;
    virtual ~gcPopMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcPopMultiMap : public std::multimap<size_t, std::pair< size_t, size_t> >
{
  public:
    gcPopMultiMap() ;
    virtual ~gcPopMultiMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcLocusMap : public std::map<size_t,gcLocus>
{
  public:
    gcLocusMap() ;
    virtual ~gcLocusMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};


class gcRegionPopIdPair : public std::pair<size_t, size_t>
{
  public:
    gcRegionPopIdPair() ;
    gcRegionPopIdPair(size_t i,size_t j);
    ~gcRegionPopIdPair() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcPanelMap : public std::map<gcRegionPopIdPair,gcPanel>
{
  public:
    gcPanelMap() ;
    virtual ~gcPanelMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcParentMap : public std::map<size_t,gcParent>
{
  public:
    gcParentMap() ;
    virtual ~gcParentMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcTraitMap : public std::map<size_t,gcTraitInfo>
{
  public:
    gcTraitMap() ;
    virtual ~gcTraitMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcAlleleMap : public std::map<size_t,gcTraitAllele>
{
  public:
    gcAlleleMap() ;
    virtual ~gcAlleleMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcPhenoMap : public std::map<size_t,gcPhenotype>
{
  public:
    gcPhenoMap() ;
    virtual ~gcPhenoMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcStringMap : public std::map<size_t,wxString>
{
  public:
    gcStringMap() ;
    virtual ~gcStringMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcMigrationPair : public std::pair<size_t, size_t>
{
  public:
    gcMigrationPair() ;
    gcMigrationPair(size_t i,size_t j);
    virtual ~gcMigrationPair() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

class gcMigrationMap : public std::map<gcMigrationPair,gcMigration>
{
  public:
    gcMigrationMap() ;
    virtual ~gcMigrationMap() ;
    void DebugDump(wxString prefix=wxEmptyString) const;
};

typedef std::vector<GCQuantum *>        objVector;
typedef std::vector<const GCQuantum *>  constObjVector;

class constBlockVector : public std::vector<const GCParseBlock *>
{
  public:
    constBlockVector() ;
    virtual ~constBlockVector() ;
    //void DebugDump(wxString prefix=wxEmptyString) const;
};

class popVector : public std::vector<gcPopulation *>
{
  public:
    popVector() ;
    virtual ~popVector() ;
};

class locVector : public std::vector<gcLocus *>
{
  public:
    locVector() ;
    virtual ~locVector() ;
};

#endif  // GC_STRUCTURE_MAPS_H

//____________________________________________________________________________________
