// $Id: IndividualDS.cpp,v 1.17 2011/03/07 06:08:48 bobgian Exp $

/*
  Copyright 2002 Patrick Colacurcio, Peter Beerli, Mary Kuhner,
  Jon Yamato and Joseph Felsenstein

  This software is distributed free of charge for non-commercial use
  and is copyrighted.  Of course, we do not guarantee that the software
  works, and are not responsible for any damage you may cause or have.
*/

#include <cassert>
#include "Converter_IndividualDS.h"
#include "Converter_DataSourceException.h"
#include "stringx.h"

#ifdef DMALLOC_FUNC_CHECK
#include "/usr/local/include/dmalloc.h"
#endif

using namespace std;

//------------------------------------------------------------------------------------

IndividualDS::IndividualDS (const string& name,
                            const Sequence& seq)
    : m_name(name)
{
    m_seq.push_back(seq);
}                               // IndividualDS::IndividualDS

//------------------------------------------------------------------------------------

IndividualDS::IndividualDS (const string& name,
                            const string& seq)
    : m_name(name)
{
    m_seq.push_back(Sequence(seq));
}                               // IndividualDS::IndividualDS

//------------------------------------------------------------------------------------

IndividualDS::IndividualDS (const string& name,
                            const string& seq,
                            const string& dataType)
    : m_name(name)
{
    m_seq.push_back(Sequence(seq, dataType, name));
}                               // IndividualDS::IndividualDS

//------------------------------------------------------------------------------------

IndividualDS::~IndividualDS ()
{
}                               // IndividualDS::~IndividualDS

//------------------------------------------------------------------------------------

string
IndividualDS::getSequence() const
{
    return m_seq[0].asString();
}

//------------------------------------------------------------------------------------

int
IndividualDS::getSequenceLength() const
{
    return m_seq[0].getSequenceLength();
}

//------------------------------------------------------------------------------------

vector<string> IndividualDS::GetAllSeqNames() const
{
    vector<string> seqnames;

    vector<Sequence>::const_iterator seq;
    for(seq = m_seq.begin(); seq != m_seq.end(); ++seq)
    {
        seqnames.push_back(seq->GetName());
    }

    return seqnames;

} // IndividualDS::GetAllSeqNames

//------------------------------------------------------------------------------------

vector<string> IndividualDS::GetHapNames() const
{
    return m_seqnames;
} // IndividualDS::GetHapNames

//------------------------------------------------------------------------------------

string IndividualDS::GetHapNamesForPrint() const
{
    string names;
    vector<string>::const_iterator name;
    for(name = m_seqnames.begin(); name != m_seqnames.end(); ++name)
        names += *name + ",";

    return names;
} // IndividualDS::GetHapNamesForPrint

//------------------------------------------------------------------------------------

bool IndividualDS::HasSequence(const string& seqname) const
{
    vector<Sequence>::const_iterator seq;
    for(seq = m_seq.begin(); seq != m_seq.end(); ++seq)
    {
        if (seq->IsNamed(seqname)) return true;
    }

    return false;

} // IndividualDS::HasSequence

//------------------------------------------------------------------------------------

bool IndividualDS::HasNoSequences() const
{
    return m_seq.empty();

} // IndividualDS::HasNoSequences

//------------------------------------------------------------------------------------

bool IndividualDS::HasNonContiguousData() const
{
    vector<Sequence>::const_iterator seq;
    for(seq = m_seq.begin(); seq != m_seq.end(); ++seq)
    {
        if (seq->HasNonContiguousData())
            return true;
    }

    return false;

} // IndividualDS::HasNonContiguousData

//------------------------------------------------------------------------------------

bool IndividualDS::HasSNPs() const
{
    vector<Sequence>::const_iterator seq;
    for(seq = m_seq.begin(); seq != m_seq.end(); ++seq)
    {
        if (seq->HasSNPs())
            return true;
    }

    return false;

} // IndividualDS::HasSNPs

//------------------------------------------------------------------------------------

Sequence IndividualDS::PopSequence(const string& seqname)
{
    vector<Sequence>::iterator seq;
    for(seq = m_seq.begin(); seq != m_seq.end(); ++seq)
    {
        if (seq->IsNamed(seqname))
        {
            Sequence sequence(*seq);
            m_seq.erase(seq);
            return sequence;
        }
    }

    assert(false); // Shouldn't be able to get here

    Sequence sequence(m_seq.front());
    return sequence;

} // IndividualDS::PopSequence

//------------------------------------------------------------------------------------

string
IndividualDS::getName() const
{
    return m_name;
}

//------------------------------------------------------------------------------------

void
IndividualDS::setName(const string& name)
{
    m_name = name;
}

//------------------------------------------------------------------------------------

string
IndividualDS::getXML(unsigned int numTabs) const
{
    string individualXML;

    addTabs(numTabs, individualXML);
    individualXML += "<individual name=\"" + m_name + "\">\n";

    ++numTabs;
    if (!m_unknownphase.empty())        // if we have phase unknown data
    {
        addTabs(numTabs, individualXML);
        individualXML += "<phase type=\"unknown\">";
        vector<long>::const_iterator marker;
        for(marker = m_unknownphase.begin();
            marker != m_unknownphase.end();
            ++marker)
        {
            individualXML += " " + ToString(*marker);
        }
        individualXML += " </phase>\n";
    }

    vector<Sequence>::const_iterator seqit;
    for(seqit = m_seq.begin(); seqit != m_seq.end(); ++seqit)
        individualXML += seqit->getXML(numTabs);

    --numTabs;
    addTabs(numTabs, individualXML);
    individualXML += "</individual>\n";

    return individualXML;
}

//____________________________________________________________________________________
